#!/usr/bin/env python3
# Time-stamp: "2021-09-20 23:09:06 (ywatanabe)"

from . import dsp
from . import general
from . import ml
from . import plt
from . import stats
from . import resource


__copyright__ = "Copyright (C) 2021 Yusuke Watanabe"
__version__ = "0.0.15"
__license__ = "GPL3.0"
__author__ = "ywatanabe1989"
__author_email__ = "ywata1989@gmail.com"
__url__ = "https://github.com/ywatanabe1989/mngs"
